function  [cpdag] = dag_to_cpdag(dag)
% DAG_TO_CPDAG produce a N*N matrix which values respect :
%
% 	If the edge is compelled then 1 on the edge.
%	If the edge is reversible then 1 on the edge and 1 in the reverse edge.
%
% Make sure that the entry is a DAG.
%
% See D.M. Chickering: "Learning Equivalence Classes of Bayesian-Network Structures".
%
% % philippe.leray@insa-rouen.fr,  olivier.francois@insa-rouen.fr

cpdag = abs(label_edges(dag));

%%==============================================================================

function [label] = label_edges(dag)
% LABEL-EDGES produce a N*N matrix which values are
% 	+1 if the edge is compelled or
%	-1 if the edge is reversible.
% Make sure that the entry is a DAG.
%
% olivier.francois@insa-rouen.fr

N=length(dag);
[order xedge yedge] = order_edges(dag);
label = 2*dag;

NbEdges = length(xedge) ;

for Edge=1:NbEdges,
    xlow=xedge(Edge);
    ylow=yedge(Edge);
    if label(xlow,ylow)==2
        fin = 0;
        wcompelled = find(label(:,xlow)==1);
        parenty = find(label(:,ylow)~=0);

        for w = wcompelled
            if ~ismember(w,parenty)
                label(parenty,ylow)=1;
                fin = 1;
            elseif fin == 0
                label(w,ylow)=1;
            end
        end
        if fin == 0
            parentx = [xlow ; find(label(:,xlow)~=0)];
            if ~isempty(mysetdiff(parenty,parentx))
                label(find(label(:,ylow)==2),ylow)=1;
            else	
                label(xlow,ylow)=-1;
                label(ylow,xlow)=-1;
                label(find(label(:,ylow)==2),ylow)=-1;
                label(ylow,find(label(:,ylow)==2))=-1;
            end
        end
    end
end

%%%========================================================================================
function [order, x, y] = order_edges(dag)
% ORDER_EDGES produce a total (natural) ordering over the edges in a DAG.
% Make sure that the entry is a DAG.
%
% philippe.leray@insa-rouen.fr, olivier.francois@insa-rouen.fr
%
% 5 mars 2003

if acyclic(dag)==0
    error('Requires an acyclic graph');
end

N=length(dag);
order = zeros(N,N);

node_order = topological_sort(dag);
[tmp oo] = sort(node_order);

dag=dag(node_order,node_order);
[x y]=find(flipud(dag)==1);
nb_edges=length(x);
order(sub2ind([N N],N+1-x,y))=1:nb_edges ;

order=order(oo,oo);
x=node_order(N+1-x);
y=node_order(y);

